/* (non-javadoc)
 * BioAssay_point.java is part of BASEFile.
 *
 * BASEFile is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * BASEFile is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with BASEFile; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * johan.enell@onk.lu.se
 * Johan Enell, Dept Oncology, Lund University, S-221 85 Lund, Sweden
 */
package mergebioassay;

/**
 * A simple class that wraps a BioAssay_point. It holds the basic values a BioAssay_point 
 * has, which is reporter, position, intenisty1 and intensity2.  
 * 
 * @author Johan Enell
 */
public class BioAssay_point {
  public int reporter, position;
  public float int1, int2;

  /**
   * Will constuct a new BioAssay_point.
   * 
   * @param position the position of the point
   * @param reporter the reorter id of the point
   * @param int1 intensity 1 of the point
   * @param int2 intensity 2 of the point
   */
  public BioAssay_point(int position, int reporter, float int1, float int2) {
    this.position = position;
    this.reporter = reporter;
    this.int1 = int1;
    this.int2 = int2;
  }

  /**
   * Will calculate the ration of the point
   * 
   * @return the ratio of the point, -1 if one of the intensities is 0 or less.
   */
  public float ratio() {
    return (int1 > 0 && int2 > 0) ? int1 / int2 : -1;
  }
}
