/* (non-javadoc)
 * NoGroup.java is part of Merge.
 *
 * Merge is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Merge is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Merge; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * johan.enell@onk.lu.se
 * Johan Enell, Dept Oncology, Lund University, S-221 85 Lund, Sweden
 */

package mergebioassay;

import java.io.IOException;
import java.util.List;
import java.util.ArrayList;

import mergebioassay.mergers.Merge_assay;

import basefile.BASEFileException;
import basefile.BASEFileReader;
import basefile.BASEFileSection;

/**
 * A simple utility class to handle the assyas that doesn't fit in any group. It is basically 
 * a List with a print method added.
 * 
 * @author Johan Enell
 */
public class NoGroup extends ArrayList<Merge_assay>
{

	/**
	 * A method that will print the assays as a "spots" section on System.out.
	 * 
	 * @param section - the spots section to read
	 * @param reader - the baseFile where the data is located
	 */
	public void printNoGroup(BASEFileSection section, BASEFileReader reader)
	{
		int intensity1Pos = -1;
		int intensity2Pos = -1;
		int assayDataPos = -1;
		int positionPos = -1;
		int reporterPos = -1;

		int nbrOfColumns = -1;

		List<String> assayFieldsMap;
		List<String> assaysMap;
		List<String> columnsMap;

		assaysMap = section.findFieldList("assays");
		if (assaysMap == null)
		{
			System.err.print("assaysMap row in spots section not found\n");
			System.exit(0);
		}

		columnsMap = section.findFieldList("columns");
		if (columnsMap == null)
		{
			System.err.print("Columns row in spots section not found\n");
			System.exit(0);
		}

		assayFieldsMap = section.findFieldList("assayFields");
		if (assayFieldsMap == null)
		{
			System.err.print("assayFields row in spots section not found\n");
			System.exit(0);
		}
		nbrOfColumns = (columnsMap.size() - 1) + (assayFieldsMap.size() * assaysMap.size());

		positionPos = columnsMap.indexOf("position");
		if (positionPos == -1)
		{
			System.err.print("position column in columns row missing\n");
			System.exit(0);
		}

		reporterPos = columnsMap.indexOf("reporter");
		if (reporterPos == -1)
		{
			System.err.print("reporter column in columns row missing\n");
			System.exit(0);
		}

		intensity1Pos = assayFieldsMap.indexOf("intensity1");
		if (intensity1Pos == -1)
		{
			System.err.print("intensity1 field in assayFields row missing\n");
			System.exit(0);
		}

		intensity2Pos = assayFieldsMap.indexOf("intensity2");
		if (intensity2Pos == -1)
		{
			System.err.print("intensity2 field in assayFields row missing\n");
			System.exit(0);
		}

		assayDataPos = columnsMap.indexOf("assayData");
		if (assayDataPos == -1)
		{
			System.err.print("assayData column in columns row missing\n");
			System.exit(0);
		}

		System.out.println();
		System.out.print("section\tspots\n");
		System.out.print("columns\tposition\treporter\tassayData\n");
		System.out.print("assayFields\tintensity1\tintensity2\tJE_nbrOfElements\n");
		System.out.print("setExtraFloats\tJE_nbrOfElements\n");

		System.out.print("assays");
		for (int i = 0; i < size(); i++)
		{
			BioAssay tmp = get(i);
			System.out.print("\t" + tmp.id);
		}

		System.out.print("\n%\n");

		try
		{
			String[] row = reader.readDataRow(nbrOfColumns);
			float int1 = 0, int2 = 0;

			while (row != null)
			{
				System.out.print(row[positionPos] + "\t" + row[reporterPos]);
				for (int i = 0; i < size(); i++)
				{
					BioAssay tmp = get(i);
					int assay = assaysMap.indexOf(tmp.id);
					try
					{
						int1 = Float.parseFloat(row[assayDataPos + intensity1Pos + assay * assayFieldsMap.size()]);
						int2 = Float.parseFloat(row[assayDataPos + intensity2Pos + assay * assayFieldsMap.size()]);
						System.out.print("\t" + int1 + "\t" + int2 + "\t1");
					}
					catch (NumberFormatException nfe)
					{
						System.out.print("\t\t\t0");
					}
				}
				System.out.print("\n");
				row = reader.readDataRow(nbrOfColumns);
			}
		}
		catch (BASEFileException bfe)
		{
			bfe.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
}
