/* (non-javadoc)
 * ArithmeticIntensityMerger.java is part of Merge.
 *
 * Merge is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Merge is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Merge; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * johan.enell@onk.lu.se
 * Johan Enell, Dept Oncology, Lund University, S-221 85 Lund, Sweden
 */

package mergebioassay.mergers;

import mergebioassay.BioAssay;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import basefile.BASEFileReader;
import basefile.BASEFileSection;
import basefile.BadSectionException;

/**
 * @author Johan Enell
 * 
 * The ArithmeticIntensityMerger implements the abstract merger class. It will
 * use the arithmetic mean of the intensities.
 */
public class ArithmeticIntensityMerger extends Merger
{

	/*
	 * (non-Javadoc)
	 * 
	 * @see mergers.Merger#merge(baseFile.BASEFileSection,
	 *      baseFile.BASEFileReader, java.util.HashMap)
	 */
	@Override
	public void merge(BASEFileSection section, BASEFileReader reader, HashMap<String, List<Merge_assay>> assayGroups)
	{
		int intensity1Pos = -1;
		int intensity2Pos = -1;

		super.init(section, reader);

		intensity1Pos = assayFieldsMap.indexOf("intensity1");
		if (intensity1Pos == -1)
		{
			System.err.print("intensity1 field in assayFields row missing\n");
			System.exit(0);
		}

		intensity2Pos = assayFieldsMap.indexOf("intensity2");
		if (intensity2Pos == -1)
		{
			System.err.print("intensity2 field in assayFields row missing\n");
			System.exit(0);
		}

		try
		{

			System.out.print("section\tspots\n");
			System.out.print("columns\tposition\treporter\tassayData\n");
			System.out.print("assayFields\tintensity1\tintensity2\tJE_nbrOfElements\n");
			System.out.print("setExtraFloats\tJE_nbrOfElements\n");
			System.out.print("assays");
			for (int i = 0; i < assayGroups.size(); i++)
				System.out.print("\t" + (i + 1));
			System.out.print("\n%\n");
			String[] row = reader.readDataRow(nbrOfColumns);

			Iterator it = assayGroups.keySet().iterator();
			String key;
			while (row != null)
			{
				System.out.print(row[positionPos] + "\t" + row[reporterPos]);

				it = assayGroups.keySet().iterator();
				while (it.hasNext())
				{
					key = (String) it.next();
					List<Merge_assay> micro = assayGroups.get(key);
					double int1 = 0, int2 = 0;
					DoubleVector intVector1 = new DoubleVector();
					DoubleVector intVector2 = new DoubleVector();
					for (int i = 0; i < micro.size(); i++)
					{
						BioAssay tmp = micro.get(i);
						int assay = assaysMap.indexOf(tmp.id);
						try
						{
							int1 = Double.parseDouble(row[assayDataPos + intensity1Pos + assay * assayFieldsMap.size()]);
							int2 = Double.parseDouble(row[assayDataPos + intensity2Pos + assay * assayFieldsMap.size()]);

							intVector1.add(int1);
							intVector2.add(int2);
						}
						catch (NumberFormatException nfe)
						{
							missingValues++;
						}
					}

					System.out.print("\t");
					if (intVector1.size() > 0)
					{
						System.out.print((float) arithmeticMean(intVector1.toArray()));
						System.out.print("\t");
						System.out.print((float) arithmeticMean(intVector2.toArray()));
					}
					else System.out.print("\t");
					if (intVector1.size() > 0) System.out.print("\t" + intVector1.size());
					else System.out.print("\t");
				}
				System.out.print("\n");
				row = reader.readDataRow(nbrOfColumns);
			}
			System.out.print("\n");
		}
		catch (BadSectionException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
			System.exit(0);
		}
	}
}
