/* (non-javadoc)
 * Merge_assay.java is part of Merge.
 *
 * Merge is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Merge is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Merge; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * johan.enell@onk.lu.se
 * Johan Enell, Dept Oncology, Lund University, S-221 85 Lund, Sweden
 */
package mergebioassay.mergers;

import mergebioassay.BioAssay;

import java.util.Vector;


/**
 * @author Johan Enell
 * 
 * An subclass of BioAssays. It implements a parse methode used to goup BioAssays
 */
public class Merge_assay extends BioAssay
{
	// v[0-9] - reserved for variabels %0 - %9
	// v[10] - reserved for annotation value
	// v[11] - reserved for ratio value
	public String[] v = new String[12];

	/**
	 * Creates a new Merge_assay.
	 * 
	 * @param id - the id of the BioAssay
	 * @param name - the name of the BioAssay
	 */
	public Merge_assay(String id, String name)
	{
		super(id, name);
	}

	/**
	 * This method will parse the name of the BioAssay using the input string as pattern. The
	 * pattern consists of variable numbers, %0-%9, and some characters dividing them.
	 * 
	 * @param pattern - the pattern used for the parsing
	 * @return tru if the parsing was done without any error
	 */
	public boolean parse_name(String pattern) throws BadPatternException
	{
		Vector<String> tokens = new Vector<String>();
		int start = 0;

		for (int i = pattern.indexOf('%', start); i != -1; i = pattern.indexOf('%', start))
		{
			tokens.add(pattern.substring(start, i));
			if (pattern.charAt(i + 1) >= 48 && pattern.charAt(i + 1) <= 57)
			{
				tokens.add("%" + pattern.charAt(i + 1));
			}
			else
			{
				throw new BadPatternException("A % in the assayname must be followed by a number 0-9");
			}
			start = i + 2;
		}
		if (start < pattern.length()) tokens.add(pattern.substring(start, pattern.length()));

		start = 0;
		int end = 0;
		for (int i = 0; i < tokens.size(); i++)
		{
			String tok = tokens.get(i);

			if (tok.startsWith("%"))
			{
				if (i < tokens.size() - 1)
				{
					end = name.indexOf(tokens.get(i + 1), start);

					if (end == -1) { return false; }
				}
				else
				{
					end = name.length();
				}

				v[tok.charAt(1) - 48] = name.substring(start, end);
				start = end;
			}
			else if (name.substring(start, start + tok.length()).equals(tok))
			{
				start += tok.length();
			}
			else
			{
				return false;
			}
		}
		return true;
	}
}
