/* (non-javadoc)
 * RatioRatioMerger.java is part of Merge.
 *
 * Merge is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Merge is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Merge; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * johan.enell@onk.lu.se
 * Johan Enell, Dept Oncology, Lund University, S-221 85 Lund, Sweden
 */
package mergebioassay.mergers;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import basefile.BASEFileReader;
import basefile.BASEFileSection;
import basefile.BadSectionException;

/**
 * @author Johan Enell
 * 
 * The RatioRatioMerger implements the abstract merger class. It will calculate
 * as ratio over ratio.
 */
public class RatioRatioMerger extends Merger
{
	String ratioNumerator;

	String ratioDenominator;

	/**
	 * This will create a new RartioRatioMerger.
	 * 
	 * @param ratioNumerator - the string used as numerator
	 * @param ratioDenominator - the string used as denominator
	 */
	public RatioRatioMerger(String ratioNumerator, String ratioDenominator)
	{
		this.ratioDenominator = ratioDenominator;
		this.ratioNumerator = ratioNumerator;
	}

	/* (non-Javadoc)
	 * @see mergers.Merger#merge(baseFile.BASEFileSection, baseFile.BASEFileReader, java.util.HashMap)
	 */
	@Override
	public void merge(BASEFileSection section, BASEFileReader reader, HashMap<String, List<Merge_assay>> assayGroups)
	{
		int intensity1Pos = -1;
		int intensity2Pos = -1;

		super.init(section, reader);

		intensity1Pos = assayFieldsMap.indexOf("intensity1");
		if (intensity1Pos == -1)
		{
			System.err.print("intensity1 field in assayFields row missing\n");
			System.exit(0);
		}

		intensity2Pos = assayFieldsMap.indexOf("intensity2");
		if (intensity2Pos == -1)
		{
			System.err.print("intensity2 field in assayFields row missing\n");
			System.exit(0);
		}

		try
		{
			System.out.print("section\tspots\n");
			System.out.print("columns\tposition\treporter\tassayData\n");
			System.out.print("assayFields\tintensity1\tintensity2\tJE_nbrOfElements\n");
			System.out.print("setExtraFloats\tJE_nbrOfElements\n");
			System.out.print("assays");
			for (int i = 0; i < assayGroups.size(); i++)
				System.out.print("\t" + (i + 1));
			System.out.print("\n%\n");

			String[] row = reader.readDataRow(nbrOfColumns);

			Iterator it = assayGroups.keySet().iterator();
			String key;
			while (row != null)
			{
				System.out.print(row[positionPos] + "\t" + row[reporterPos]);

				it = assayGroups.keySet().iterator();
				while (it.hasNext())
				{
					key = (String) it.next();
					List<Merge_assay> micro = assayGroups.get(key);
					double int1 = 0, int2 = 0;
					try
					{
						for (int i = 0; i < micro.size(); i++)
						{
							Merge_assay tmp = micro.get(i);
							int assay = assaysMap.indexOf(tmp.id);
							if (tmp.v[11].equals(ratioNumerator))
							{
								int1 = Double.parseDouble(row[assayDataPos + intensity1Pos + assay * assayFieldsMap.size()]);
								int1 /= Double.parseDouble(row[assayDataPos + intensity2Pos + assay * assayFieldsMap.size()]);
							}
							else if (tmp.v[11].equals(ratioDenominator))
							{
								int2 = Double.parseDouble(row[assayDataPos + intensity1Pos + assay * assayFieldsMap.size()]);
								int2 /= Double.parseDouble(row[assayDataPos + intensity2Pos + assay * assayFieldsMap.size()]);
							}
						}
						System.out.print("\t" + int1 + "\t" + int2 + "\t2");
					}
					catch (NumberFormatException nfe)
					{
						System.out.print("\t\t\t0");
						missingValues++;
					}
				}
				System.out.print("\n");
				row = reader.readDataRow(nbrOfColumns);
			}
			System.out.print("\n");
		}
		catch (BadSectionException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
			System.exit(0);
		}
	}
}
